/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package sayitsmart;

/*
   Copyright (c) 1999-2005 Audium Corporation
   All rights reserved
*/

import com.audium.server.sayitsmart.*;
import java.util.*;

/**
 * This Plugin is used to Access Dates in Hijra and return
 * 1. Month name in Hijra
 * 2. Month Number in Arabic
 * 3. Month number in English
 */
public class HijraSayItSmart extends SayItSmartBase implements SayItSmartPlugin
{
	/* Input format real names. Here, we define an input format that handles just
	three input date formats */

	public static final String DD_MM_YYYY_INPUT_FORMAT = "DD-MM-YYYY";
	public static final String YYYY_MM_DD_INPUT_FORMAT = "YYYY-MM-DD";
	public static final String YYYYMMDD_INPUT_FORMAT = "YYYYMMDD";

	/* Output format real names. Here, we define 3 outputs namely the hijra month name,
           the month number in arabic and in english. */

	public static final String NAME_OUTPUT_FORMAT = "name";
	public static final String NUMBER_AR_OUTPUT_FORMAT = "number_a";
	public static final String NUMBER_EN_OUTPUT_FORMAT = "number_e";


	/* Fileset real names. Here, we define three filesets each for each output format*/

	public static final String NAME_FILESET = "name";
	public static final String NUMBER_AR_FILESET = "number_a";
	public static final String NUMBER_EN_FILESET = "number_e";

	/* Input format, output format, and fileset display names.   */

	public static final String DD_MM_YYYY_INPUT_FORMAT_DISPLAY = "Date Format: DD-MM-YYYY";
	public static final String YYYY_MM_DD_INPUT_FORMAT_DISPLAY = "Date Format: YYYY-MM-DD";
	public static final String YYYYMMDD_INPUT_FORMAT_DISPLAY = "Date Format: YYYYMMDD";

	public static final String NAME_OUTPUT_FORMAT_DISPLAY = "Hijra Month Name";
	public static final String NUMBER_AR_OUTPUT_FORMAT_DISPLAY = "Month Number in Arabic";
	public static final String NUMBER_EN_OUTPUT_FORMAT_DISPLAY = "Month Number in English";

	public static final String NAME_FILESET_DISPLAY = "Hijra Month Name";
	public static final String NUMBER_AR_FILESET_DISPLAY = "Month No. in Arabic";
	public static final String NUMBER_EN_FILESET_DISPLAY = "Month No. in English";

	/* Input format, output format, and fileset descriptions. */

	public static final String DD_MM_YYYY_INPUT_FORMAT_DESCRIPTION = "The input is a date of the format DD-MM-YYYY were DD is the date, MM is the Month and YYYY is the year. The date is in Hijra.";
	public static final String YYYY_MM_DD_INPUT_FORMAT_DESCRIPTION = "The input is a date of the format YYYY-MM-DD were DD is the date, MM is the Month and YYYY is the year. The date is in Hijra.";
	public static final String YYYYMMDD_INPUT_FORMAT_DESCRIPTION = "The input is a date of the format YYYYMMDD were DD is the date, MM is the Month and YYYY is the year. The date is in Hijra.";

	public static final String NAME_OUTPUT_FORMAT_DESCRIPTION = "The output will be Hijra Month Name";
	public static final String NUMBER_AR_OUTPUT_FORMAT_DESCRIPTION = "The output will be Hijra Month Number in Arabic.";
	public static final String NUMBER_EN_OUTPUT_FORMAT_DESCRIPTION = "The output will be Hijra Month Number in English.";

	public static final String NAME_FILESET_DESCRIPTION = "This fileset speaks the Hijra Month Name.";
	public static final String NUMBER_AR_FILESET_DESCRIPTION = "This fileset speaks the Hijra Month Number in Arabic.";
	public static final String NUMBER_EN_FILESET_DESCRIPTION = "This fileset speaks the Hijra Month number in English.";

	/* We will create a static hashMap that contains the months that we look
	for. We keep a String array in each key which holds the audio file name of
	the months. The different audio file
	names are figured out dynamically. */

	public static HashMap MONTHS = new HashMap();

	static {
	    MONTHS.put(new String("01"), new String[] {"01",   "Muharram",       "Waahid",     "One"});
	    MONTHS.put(new String("02"), new String[] {"02",   "Safar",          "Ithnaan",    "Two"});
	    MONTHS.put(new String("03"), new String[] {"03",   "RabeeyyulAwwal", "Thalaatha",  "Three"});
	    MONTHS.put(new String("04"), new String[] {"04",   "RabeeyyuThaani", "ArbaAh",     "Four"});
	    MONTHS.put(new String("05"), new String[] {"05",   "JamaadulAwwal",  "KhamsAh",    "Five"});
	    MONTHS.put(new String("06"), new String[] {"06",   "JamaaduThaani",  "SithAh",     "Six"});
	    MONTHS.put(new String("07"), new String[] {"07",   "Rajab",          "SabuAh",     "Seven"});
	    MONTHS.put(new String("08"), new String[] {"08",   "Shaiban",        "ThamaaNiya", "Eight"});
	    MONTHS.put(new String("09"), new String[] {"09",   "Ramadhan",       "TisAh",      "Nine"});
	    MONTHS.put(new String("10"), new String[] {"10",   "Shawwaal",       "Ashara",     "Ten"});
	    MONTHS.put(new String("11"), new String[] {"11",   "DulQaidah",      "AhdaAshar",  "Eleven"});
	    MONTHS.put(new String("12"), new String[] {"12",   "DulHijja",       "IthnaAshar", "Twelve"});
	};

    /**
     * This method is used to specify the information on all parts of the plugin
     * to Builder for Studio. We define the 3 input formats, 3 output formats,
     * and 3 filesets here along with their display name and description.
     */
	public SayItSmartDisplay getDisplayInformation() throws SayItSmartException
	{
    	String description = "This Plugin Returns the Hijra Month in the given Hijra Date";

		SayItSmartDisplay toReturn = new SayItSmartDisplay("hijra", "Hijra Month", description);
		toReturn.addInputFormat(DD_MM_YYYY_INPUT_FORMAT, DD_MM_YYYY_INPUT_FORMAT_DISPLAY, DD_MM_YYYY_INPUT_FORMAT_DESCRIPTION);
		toReturn.addInputFormat(YYYY_MM_DD_INPUT_FORMAT, YYYY_MM_DD_INPUT_FORMAT_DISPLAY, YYYY_MM_DD_INPUT_FORMAT_DESCRIPTION);
		toReturn.addInputFormat(YYYYMMDD_INPUT_FORMAT, YYYYMMDD_INPUT_FORMAT_DISPLAY, YYYYMMDD_INPUT_FORMAT_DESCRIPTION);

		toReturn.addOutputFormat(NAME_OUTPUT_FORMAT, NAME_OUTPUT_FORMAT_DISPLAY, NAME_OUTPUT_FORMAT_DESCRIPTION);
		toReturn.addOutputFormat(NUMBER_AR_OUTPUT_FORMAT, NUMBER_AR_OUTPUT_FORMAT_DISPLAY, NUMBER_AR_OUTPUT_FORMAT_DESCRIPTION);
		toReturn.addOutputFormat(NUMBER_EN_OUTPUT_FORMAT, NUMBER_EN_OUTPUT_FORMAT_DISPLAY, NUMBER_EN_OUTPUT_FORMAT_DESCRIPTION);

		toReturn.addFileset(NAME_FILESET, NAME_FILESET_DISPLAY, NAME_FILESET_DESCRIPTION);
		toReturn.addFileset(NUMBER_AR_FILESET, NUMBER_AR_FILESET_DISPLAY, NUMBER_AR_FILESET_DESCRIPTION);
		toReturn.addFileset(NUMBER_EN_FILESET, NUMBER_EN_FILESET_DISPLAY, NUMBER_EN_FILESET_DESCRIPTION);

		return toReturn;
    }

    /**
     * Here, we handle the input and output format dependencies.
     */
    public SayItSmartDependency getFormatDependencies() throws SayItSmartException
    {
        SayItSmartDependency toReturn = new SayItSmartDependency(DD_MM_YYYY_INPUT_FORMAT, new String[] {NAME_OUTPUT_FORMAT, NUMBER_AR_OUTPUT_FORMAT, NUMBER_EN_OUTPUT_FORMAT});
        toReturn.addParent(YYYY_MM_DD_INPUT_FORMAT, new String[] {NAME_OUTPUT_FORMAT, NUMBER_AR_OUTPUT_FORMAT, NUMBER_EN_OUTPUT_FORMAT});
        toReturn.addParent(YYYYMMDD_INPUT_FORMAT, new String[] {NAME_OUTPUT_FORMAT, NUMBER_AR_OUTPUT_FORMAT, NUMBER_EN_OUTPUT_FORMAT});
        return toReturn;
    }

    /**
     * Here, we handle the output format and filset dependencies.
     */
    public SayItSmartDependency getFilesetDependencies() throws SayItSmartException
    {
        SayItSmartDependency toReturn = new SayItSmartDependency(NAME_OUTPUT_FORMAT, new String[] {NAME_FILESET});
        toReturn.addParent(NUMBER_AR_OUTPUT_FORMAT, new String[] {NUMBER_AR_FILESET});
        toReturn.addParent(NUMBER_EN_OUTPUT_FORMAT, new String[] {NUMBER_EN_FILESET});
        return toReturn;
    }

    /**
     * This method actually does the convertion. It converts the month into a list
     * of audio files with TTS backups all encapsulated into a single
     * SayItSmartContent class. Errors are thrown if the data is incorrect or
     * the input format, output format, or filset is invalid.
     */
    public SayItSmartContent convertToFiles(Object dataAsObject, String inputFormat, String outputFormat, String fileset) throws SayItSmartException
    {
        /* This is a utility class defined in the SayItSmartBase class that
        checks if the data, input format, output format, and fileset all conform
        to the expected vaues. If not, it throws an exception that propagates
        its way out. */
        validateArguments(dataAsObject, inputFormat, outputFormat, fileset);

        /* Even though Say It Smart plugins have the capability to accept any
        Java Object, we are only interested in Strings, so we cast to a String
        accordingly. We also convert the string to a lowercase because upper
        and lowercase letters are spelled the same and we don't want to have to
        add separate options in the HashMap for both. */
        String data = dataAsObject.toString().toLowerCase();

        SayItSmartContent toReturn = new SayItSmartContent();

        int index;
        String outputPostfix = "";
        /* We make sure the output format is handled correctly. If we are using
        the name output, we set the index of the String array to 1, which
        contains the TTS backup. If it is Arabic number output format, we use
        index 2. If it is English Number we use index 3 */
        if (outputFormat.compareTo(NAME_OUTPUT_FORMAT) == 0) {
            index = 1; // index in Hash Map
            outputPostfix = "_name";
        }
        else if (outputFormat.compareTo(NUMBER_AR_OUTPUT_FORMAT) == 0){
            index = 2;
            outputPostfix = "_num_ar";
        }
        else {
            index = 3;
            outputPostfix = "_num_en";
        }
        String month = "";
        if(inputFormat.equals(DD_MM_YYYY_INPUT_FORMAT)){
            month = data.substring(3, 5);
        }
        else if(inputFormat.equals(YYYY_MM_DD_INPUT_FORMAT)){
            month = data.substring(5, 7);
        }
        else if(inputFormat.equals(YYYYMMDD_INPUT_FORMAT)){
            month = data.substring(4, 6);
        }
        else{
            throw new SayItSmartException("Say It Smart Spelling Error - The input format  \"" + inputFormat +
                                              "\" is invalid.");
        }





            /* If we could not find the month in the HashMap, then it is not
            a valid month, so we throw an exception. */
            Object wordsAsObject = MONTHS.get(new String(month));
            if (wordsAsObject == null) {
                throw new SayItSmartException("Say It Smart Month Error - The input data \"" + data +
                                              "\" must be of the Format: \"" + inputFormat +
                                              "\" .");
            }

            /* Now we add to the SayItSmartContent object we intend on
            returning. We create the filename by first taking the part listed
            in the HashMap, appending the output postfix.   */
             String[] words = (String[]) wordsAsObject;
             toReturn.add(words[0] + outputPostfix , words[index], false);


    	return toReturn;
    }

    /**
     * This method returns what files need to be recorded for each filset. At
     * this stage, this method is not used by Builder for Studio, but should be
     * provided for future compatibility. The idea would be to call this method
     * to learn what needs to be recorded. This could be combined with a list
     * of all the audio files required by the application to create a script
     * that can be given to a voice actor to record.
     */
    public String[] getFilesetContents(String fileset) throws SayItSmartException
    {
        if (fileset.compareTo(NAME_FILESET) == 0) {
            return new String[] {"01_name", "02_name", "03_name", "04_name",
                                 "05_name", "06_name",
                                 "07_name", "08_name", "09_name", "10_name",
                                 "11_name", "12_name" };
        }
        else if (fileset.compareTo(NUMBER_AR_FILESET) == 0) {
            return new String[] {"01_number_ar", "02_number_ar", "03_number_ar", "04_number_ar",
                                 "05_number_ar", "06_number_ar",
                                 "07_number_ar", "08_number_ar", "09_number_ar", "10_number_ar",
                                 "11_number_ar", "12_number_ar" };
        }
        else if (fileset.compareTo(NUMBER_EN_FILESET) == 0) {
            return new String[] {"01_number_en", "02_number_en", "03_number_en", "04_number_en",
                                 "05_number_en", "06_number_en",
                                 "07_number_en", "08_number_en", "09_number_en", "10_number_en",
                                 "11_number_en", "12_number_en" };
        }
        else {
            throw new SayItSmartException("Say It Smart Hijra Error - The fileset \"" + fileset +
                                          "\" is not supported");
        }

    }

    /**
     * Sample Use
     */
    public static void main(String[] args) throws SayItSmartException
    {
    	HijraSayItSmart hijra = new HijraSayItSmart();

    	System.out.println("RESULT: "+hijra.convertToFiles("10-10-1431",
                                                         DD_MM_YYYY_INPUT_FORMAT,
                                                         NAME_OUTPUT_FORMAT,
                                                         NAME_FILESET));
    }
}

